/* texturemappingGLUT.c
 * Exemple d'application utilisant le placage de textures. 
 * (filtrage, rpter et resserrer des textures) 
 * Touches:
 *	ESC - Quitter
 * 	p - Augmente le nombre de rptitions des textures
 *	m - Diminue le nombre de rptitions des textures
 *	x - Resserrer/rpter la texture suivant l'axe x
 *	y - Resserrer/rpter la texture suivant l'axe y
 *	f - Change la mthode de filtrage des textures
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 23 aot 2002
 */

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "tga.h"

/* Caractristiques de la fentre et de l'cran */
#define TITLE "TextureMappingGLUT (Vouriot Thierry)"	// titre du programme
#define WINDOW_WIDTH 350	// largeur de la fentre
#define WINDOW_HEIGHT 300	// largeur de la fentre
#define	WINDOW_POS_X 100	// position x de la fentre
#define WINDOW_POS_Y 100	// position y de la fentre
#define FULLSCREEN_WIDTH 640	// largeur en mode plein cran
#define FULLSCREEN_HEIGHT 480	// hauteur en mode plein cran
#define FULLSCREEN_DEPTH 16	// bit par pixels en mode plen cran

/* Dfinitions pour Windows */
#ifdef _WIN32
#include <windows.h>
#pragma comment( lib, "opengl32.lib" )			// recherche de OpenGL32.lib pendant l'dition de liens
#pragma comment( lib, "glu32.lib" )			// recherche de Glu32.lib pendant l'dition de liens
#pragma comment( lib, "glut32.lib" )			// recherche de Glut32.lib pendant l'dition de liens
#endif

/* Dfinition des boolens pour C */
#ifndef bool
#define bool unsigned char
#define true 1 == 1
#define false !true
#endif

#define ESC 27			// code ascii de la touche escape

/* Fullscreen */
static bool g_fullscreen = false;

/* Variable contenant le temps au moment de l'affichage de la
 * dernire image, cet variable permet de baser les
 * mouvements sur le temps et non pas sur la vitesse d'affichage
 * de votre ordinateur */
static int g_timeAtLastFrame = 0;

/* Textures ids */
static GLuint texName[2];
/* Images */
static TGAFILE img1;
static TGAFILE img2;

/* Angle de rotation */
static GLfloat rotY = 0.0;
/* Nombre de rptitions */
static GLfloat rep = 3.0;

/* Paramtres des textures */
static GLenum paramX = GL_REPEAT;
static GLenum paramY = GL_REPEAT;
static GLenum filter = GL_NEAREST;	

/* Prototypes */
void init(void);
void askFullscreen(void);
void glutDisplay(void);
void glutIdle(void);
void glutReshape(int w, int h);
void glutKeyboard(unsigned char key, int x, int y);
void terminateApplication(void);

/* Main */
int main(int argc, char ** argv)
{	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
	#ifdef _WIN32
	askFullscreen();
	#endif
	if (!g_fullscreen) {
		glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
		glutInitWindowPosition(WINDOW_POS_X, WINDOW_POS_Y);
		glutCreateWindow(TITLE);
	}
	init();
	glutDisplayFunc(glutDisplay);
	glutIdleFunc(glutIdle);
	glutReshapeFunc(glutReshape);
	glutKeyboardFunc(glutKeyboard);
	glutMainLoop();
	return 0;
}

/* Permet de passer en mode plein ecran ou en mode fenetre
 * Utilise GLUT_GAME_MODE disponible uniquement sous Windows
 */
#ifdef _WIN32
void askFullscreen(void)
{
	char gameMode[14];
	
	/* Demande  l'utilisateur s'il veut le mode plein cran */
	if (!g_fullscreen)
	g_fullscreen = (MessageBox(NULL, "Click Yes to go to full screen (Recommended)", "Start FullScreen?",
						MB_ICONQUESTION | MB_YESNO) == IDYES);
	
	
	if (g_fullscreen) {	// game mode (fullscreen)
		sprintf(gameMode, "%dx%d:%d", FULLSCREEN_WIDTH, FULLSCREEN_HEIGHT, FULLSCREEN_DEPTH);
		glutGameModeString(gameMode);
		// teste si le game mode est possible
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
		{
			glutEnterGameMode();
			//glutSetCursor(GLUT_CURSOR_NONE);
		}
		else {
			g_fullscreen = false;
		}
	} // mode fentr
}
#endif

/* Initialisation */
void init(void)
{
	/* Affiche des informations sur OpenGL */
	char *exten;
	exten = (char *) glGetString(GL_VENDOR);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_RENDERER);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_VERSION);
	printf("%s\n",exten);
	
	printf("\nTouches :\n");
	printf("\tp - Augmente le nombre de repetitions des textures\n");
 	printf("\tm - Diminue le nombre de repetitions des textures\n");
	printf("\tx - Resserrer/repeter la texture suivant l'axe x\n");
	printf("\ty - Resserrer/repeter la texture suivant l'axe y\n");
	printf("\tf - Change la methode de filtrage des textures\n");
	printf("\tESC - Quitter\n");
	
	g_timeAtLastFrame = glutGet(GLUT_ELAPSED_TIME);
	
	/* Initialisation d'OpenGL */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_FLAT);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);	
	
	/* Chargement de l'image 1 */
	if (LoadTGAFile("texture1.tga", &img1) != 1) {
		printf("erreur lors du chargement de la texture TGA (1)\n");
		exit(-1);
	}
	if (checkSize(img1.imageWidth) == 0 || checkSize(img1.imageHeight) == 0 ) {
		printf("erreur: la texture TGA doit avoir une largeur et une hauteur qui soient des puissances de 2\n");
		exit(-1);
	}
	/* Chargement de l'image 2 */
	if (LoadTGAFile("texture2.tga", &img2) != 1) {
		printf("erreur lors du chargement de la texture TGA (2)\n");
		exit(-1);
	}
	if (checkSize(img2.imageWidth) == 0 || checkSize(img2.imageHeight) == 0 ) {
		printf("erreur: la texture TGA doit avoir une largeur et une hauteur qui soient des puissances de 2\n");
		exit(-1);
	}
	
	glGenTextures(2, texName);
	
	// Cration de la texture 1
	glBindTexture(GL_TEXTURE_2D, texName[0]);
	glTexImage2D(GL_TEXTURE_2D, 0, img1.texFormat, img1.imageWidth, img1.imageHeight, 
			0, img1.texFormat, GL_UNSIGNED_BYTE, img1.imageData);
			
	// Cration de la texture 2
	glBindTexture(GL_TEXTURE_2D, texName[1]);
	glTexImage2D(GL_TEXTURE_2D, 0, img2.texFormat, img2.imageWidth, img2.imageHeight, 
			0, img2.texFormat, GL_UNSIGNED_BYTE, img2.imageData);	
}
		
/* Affichage et animation */
void glutDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	// Positionnement de la camra
	glTranslatef(0.0, 0.0, -2.5);
	
	// slction de la texture 1
	glBindTexture(GL_TEXTURE_2D, texName[0]);
	// paramtres de la texture 1
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, paramX);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, paramY);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);
	
	// dessin du premier carr
	glTranslatef(-0.6, 0.0, 0.0);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(0.5f, -0.5f, 0.5f);
		glTexCoord2f(0.0f, rep); glVertex3f(0.5f, 0.5f, 0.5f);	
		glTexCoord2f(rep, rep); glVertex3f(-0.5f, 0.5f, 0.5f);
		glTexCoord2f(rep, 0.0f); glVertex3f(-0.5f, -0.5f, 0.5f);
	glEnd();
	
	// slction de la texture 1
	glBindTexture(GL_TEXTURE_2D, texName[1]);
	// paramtres de la texture 1
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, paramX);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, paramY);
	
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);
	
	// dessin et animation du second carr
	glTranslatef(1.2, 0.0, 0.0);
	glRotatef(rotY, 0.0, 1.0, 0.0);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(0.5f, -0.5f, 0.5f);
		glTexCoord2f(0.0f, rep); glVertex3f(0.5f, 0.5f, 0.5f);	
		glTexCoord2f(rep, rep); glVertex3f(-0.5f, 0.5f, 0.5f);
		glTexCoord2f(rep, 0.0f); glVertex3f(-0.5f, -0.5f, 0.5f);
	glEnd();
	
	glFlush();
	glutSwapBuffers();
}

/* Fonction excut lorsque le programme n'a plus rien  faire */
void glutIdle(void)
{
	int currentTime = glutGet(GLUT_ELAPSED_TIME);
	int frameInterval = currentTime - g_timeAtLastFrame; // Temps coul entre l'affichage de 2 images
	g_timeAtLastFrame = currentTime;
	
	// On incrmente l'angle de rotation
	rotY += (float)frameInterval / 38;
	
	if (rotY >= 360.0f)
		rotY = 0.0f;
	
	// on redemande l'affichage de la scne
	glutPostRedisplay();
}

/* Redimensionnement de la fentre */
void glutReshape(int w, int h)
{
	if (h == 0) h = 1;
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0f,(GLfloat)w/(GLfloat)h, 1.0f, 30.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

/* Gestion du clavier */
void glutKeyboard(unsigned char key, int x, int y)
{
	switch(key) {
		case ESC:	// touche escape
			terminateApplication();
			break;
		case 'P':
		case 'p': if (rep < 10.0) {
				rep += 1.0;
				printf("Repetition: %f\n", rep);
			}
			break;
		case 'M':
		case 'm': if (rep > 1.0) {
				rep -= 1.0;
				printf("Repetition: %f\n", rep);
			}
			break;
		case 'X':
		case 'x': if (paramX == GL_REPEAT) {
				paramX = GL_CLAMP;
				printf("Mode de repetition sur x : GL_CLAMP\n");
			}
			else {
				paramX = GL_REPEAT;
				printf("Mode de repetition sur x : GL_REPEAT\n");
			}
			break;
		case 'Y':
		case 'y': if (paramY == GL_REPEAT) {
				paramY = GL_CLAMP; 
				printf("Mode de repetition sur y : GL_CLAMP\n");
			}
			else {
				paramY = GL_REPEAT;
				printf("Mode de repetition sur y : GL_REPEAT\n");
			}
			break;
		case 'F':
		case 'f': if (filter == GL_NEAREST) {
				filter = GL_LINEAR;
				printf("Mode de filtrage : GL_LINEAR\n");
			}
			else {
				filter = GL_NEAREST;
				printf("Mode de filtrage : GL_NEAREST\n");
			}
			break;
	}
}

/* Termine l'application */
void terminateApplication(void)
{
	#ifdef _WIN32
	if (g_fullscreen) glutLeaveGameMode();
	#endif
	printf("\nAuteur: Thierry Vouriot\nhttp://thierry.vouriot.free.fr\n");
	exit(1);
}



